==> fetch.php?media=snfit-2.2.2.tar.gz: snfit-2.2.2/src/galaxytemplate.cc <==
#include <cmath>

#include "galaxytemplate.h"
#include "datacards.h"
#include "pathutils.h"
#include "fileutils.h"
#include "snfitexception.h"

using namespace std;



GalaxyTemplate::GalaxyTemplate(const string& cardfilename) {
   LoadTemplates(cardfilename);
   if (Has_Metallicity()) 
    cerr << "metallicity" <<  endl ; 
   else 
    cerr << "NO metallicity" <<  endl ; 
   if (Has_SFR()) 
     cerr << "SFR" <<  endl ; 
   else 
     cerr << "NO SFR" <<  endl ;
   if (Has_Stellar_Mass()) 
     cerr << "stellar mass" <<  endl ; 
   else 
     cerr << "NO stellar mass" <<  endl ;
  cout << "INFO GalaxyTemplate::GalaxyTemplate done " << endl;
}


extern "C" {
  double divdifd_(const double *, const double *, const int *, const double *, const int*);
}

/*
static double HighDegreeInterpolationValue(const double& wavelength, const double& gal_type, const std::map<int,OneGalaxyTemplate>& templates) {
  int n = templates.size();
  double *xval = new double[n];
  double *fval = new double[n];
  int c = 0;
  for(std::map<int,OneGalaxyTemplate>::const_iterator it = templates.begin(); it != templates.end(); it++, c++) {
    xval[c] = it->first;
    fval[c] = log(it->second.Value(wavelength));
  }
  int interp_degree = 3;
  return exp( divdifd_(fval, xval, &n, &gal_type, &interp_degree) );
}
*/

/*
static void ResampleUsingInterpolation(Grid2DFunction& func) {
  
  double type_min = func.Xmin();
  double type_max = func.Xmax();
  
  size_t oldNx = func.NstepX();
  size_t oldNy = func.NstepY();
  
  // first get ymin and ymax from trimming 
  // i.e. discarding y values on boundaries for which all vals(x,y)=0
  size_t iymin = 0;
  while(true) {
    bool nonzero = false;
    for(size_t ix=0;ix<oldNx;ix++) {
      if(func.val(ix,iymin)!=0) {
	nonzero = true;
	break;
      }
    }
    if(nonzero) break;
    iymin++;
  }
  size_t iymax = oldNy-1;
  while(true) {
    bool nonzero = false;
    for(size_t ix=0;ix<oldNx;ix++) {
      if(func.val(ix,iymax)!=0) {
	nonzero = true;
	break;
      }
    }
    if(nonzero) break;
    iymax--;
  }
  
  cout << "INFO ResampleUsingInterpolation trimming gives Yrange  " << func.Yval(iymin) << " " << func.Yval(iymax) << endl;
  cout << "INFO ResampleUsingInterpolation initial Yrange         " << func.Yval(0) << " " << func.Yval(oldNy-1) << endl;
  
  
  int newNy = (iymax-iymin+1);
  int newNx = (type_max-type_min)*4; // to be tuned
  
  
  double *XValues  = new double[newNx];
  double *YValues  = new double[newNy];
  double **FValues = new double*[newNx];
  
  double *FValuesAtY = new double[oldNx];

  int interp_degree = 3;
  
  for(int j =0 ;j<newNy;j++) {
    YValues[j] = func.Yval(j+iymin);
  }


  for(int i=0;i<newNx;i++) {
    
    FValues[i] = new double[newNy];
    XValues[i] = type_min + i*(type_max-type_min)/(newNx-1);
    
    for(int j =0 ;j<newNy;j++) {
      
      int jinitial = j+iymin;
      // get min val to know scale
      double minval = 1.e60;
      for(size_t k=0;k<oldNx;k++) {
	const double &f = func.val(k,jinitial);
	if(f>0 && f<minval) minval=f;
      }
      
      for(size_t k=0;k<oldNx;k++) {
	if(func.val(k,jinitial)==0) // use log here
	  FValuesAtY[k] = log(minval*1.e-8);
	else
	  FValuesAtY[k] = log(func.val(k,jinitial)); 
      }
      int int_oldNx = oldNx;
      FValues[i][j] = exp( divdifd_(FValuesAtY, func.xvals(), &int_oldNx, &XValues[i], &interp_degree) );

    }
  }
  func = Grid2DFunction(newNx,newNy,(const double **)FValues,(const double *)XValues,(const double *)YValues);
  
  
  
}
*/


void GalaxyTemplate::LoadTemplates(const string& cardfilename) {
  has_sfr = false ;
  has_stellar_mass = false ;
  has_metallicity = false ;


  bool bavard = true ;
  if ( getenv ("PLEASE_BE_QUIET" ) )
    bavard = false ;

  if (bavard)
    cout << "INFO GalaxyTemplate::LoadTemplates reading " << cardfilename << endl;
  string filename = cardfilename;
  string dirname = "";
  
  if(filename=="") {
    string dirname  =  LocateFileFromCard("GALAXYTEMPLATES");
    filename = dirname+"/templates.cards";
  }else{
    dirname = DirName(filename);
  }
  if (bavard)
    cerr << "LoadTemplates : reading " << filename << endl ;
  FILE * file = fopen(filename.c_str(),"r");
  if(!file) {
    cerr << "fatal error in GalaxyTemplate::LoadTemplates(), cannot read file " << filename << endl;
    exit(-12);
  }
  double index;
  char type[512];
  char continuum_filename[512];
  char lines_filename[512];
  


  char line[4096] ;
  while (fgets(line,4096,file))
    {
      char *start = line;
      
      if (*start == '@') 
	{
	  char globkey[500];
	  char filename_key[1000] ;
	  if (sscanf(line, "@%s %s",globkey, filename_key) == 2 )
	    {  
	      if (bavard)
		cerr << "reading " << globkey << " " << filename_key << endl ;
	      string sglobkey = globkey ;
	      if ( sglobkey == "SFR_FILE" )
		{
		   
		  if (bavard) cerr << "SFR file " << filename_key << endl ;
		  General1DFunction func(dirname+"/"+string(filename_key));
		  SFR = func ;
		  has_sfr = true ;
		}
	      if ( sglobkey == "METALLICITY_FILE" )
		{
		  if (bavard) cerr << "METALLICITY file " << filename_key << endl ;
		  General1DFunction func(dirname+"/"+string(filename_key));
		  Metallicity = func ;
		  has_metallicity = true ;
		}
	      if ( sglobkey == "STELLAR_MASS_FILE" )
		{
		  if (bavard) cerr << "stellar mass file " << filename_key << endl ;
		  General1DFunction func(dirname+"/"+string(filename_key)); 
		  Stellar_Mass = func ;
		  has_stellar_mass = true ;
	      
		}
	    }
	}
      else
	{
	  if (sscanf(line,"%lf %s %s %s",&index,type,continuum_filename,lines_filename) == 4)
	    {
	      if (bavard) cout << index << " " << type << " " << continuum_filename << " " << lines_filename << endl;

	      string uu = lines_filename ;
	      if ( uu == "none")
		{
		  HasLineSpectrum = false ;
		  cerr << "No lines provided " << endl ;
		}
	      else
		HasLineSpectrum = true ;

	      {
		General1DFunction func(dirname+"/"+string(continuum_filename));
		continuum.add_slice(index,func.values(),func.xvals(),func.Nval());
	      }
	      if (HasLineSpectrum)
		{
		  General1DFunction func(dirname+"/"+string(lines_filename));
		  lines.add_slice(index,func.values(),func.xvals(),func.Nval());
		}
	    }
	}
    }

  fclose(file);
  
  // don't resample any more
  //ResampleUsingInterpolation(continuum);
  //ResampleUsingInterpolation(lines);


  cout << "INFO GalaxyTemplate::LoadTemplates done " << endl;
}

Grid2DXSlice GalaxyTemplate::ContinuumSpectrum(const double& Type) const
{
  if(isnan(Type)) {
    cout << "ERROR GalaxyTemplate::ContinuumSpectrum Type is NaN" << endl;
    abort();
    throw(SnfitException("GalaxyTemplate::ContinuumSpectrum Type isNan")); 
  }
  if(Type<continuum.Xmin() || Type>continuum.Xmax()) {
    cout << "ERROR GalaxyTemplate::ContinuumSpectrum Type " << Type << 
      " out of range " << continuum.Xmin() << " " << continuum.Xmax() << endl;
    abort();
    throw(SnfitException("GalaxyTemplate::ContinuumSpectrum Type out of range"));
  } 
  return Grid2DXSlice(continuum,Type);
}

General1DFunction GalaxyTemplate::LineSpectrum(const double& Type) const
{
  // very clean handling of Kinney Spectra that come without lines
  if (! HasLineSpectrum)
    {
      double xval[2]={1000.,2000.};
      double fval[2]={0.,0.};
      General1DFunction null_func(2,&fval[0],&xval[0]);
      return null_func ;
    }


  if(isnan(Type)) {
    cout << "ERROR GalaxyTemplate::Spectrum Type is NaN" << endl;
    abort();
    throw(SnfitException("GalaxyTemplate::LineSpectrum Type isNan")); 
  }
  if(Type<lines.Xmin() || Type>lines.Xmax()) {
    cout << "ERROR GalaxyTemplate::Spectrum Type " << Type << 
      " out of range " << lines.Xmin() << " " << lines.Xmax() << endl;
    abort();
    throw(SnfitException("GalaxyTemplate::LineSpectrum Type out of range"));
  }
 
  size_t Nx = lines.NstepY(); // c'est pas un bug
  const double *xvals = lines.yvals(); // c'est les lambda (x = age)
  const double *pxdeb = xvals ;


  Vect& buffer = const_cast<Vect&>(lines_fvals_buffer);
  if(buffer.size()!=Nx)
    buffer.allocate(Nx);
  
  double *fvals = buffer.NonConstData();
  double *pfdeb = fvals ;

  for(size_t i=0;i<Nx;i++,xvals++,fvals++)
    *fvals = lines.Value(Type,*xvals);
  //General1DFunction func(Nx,&fvals[0],&xvals[0]);
  General1DFunction func(Nx,pfdeb,pxdeb);
  
  return func;
}


double GalaxyTemplate::Value(const double& wavelength, const double& gal_type) const {
  return continuum.Value(gal_type,wavelength);
}


static double Specific_SFR(const  General1DFunction & sfr, const  General1DFunction  & stell_mass, const double  age)
{ 

 
  // integrer entre age-500 Myr et age.
  double age_min = age-500 ;
  if (age_min < 0)
    age_min = 1. ;
  double Nage = 100 ;
  double age_step = (age-age_min)/Nage ;
  double integsfr = 0 ;
  for(int i = 0 ; i <= Nage ; i++)
    {
      double t = age_min + i*age_step ;
      integsfr += sfr.Value(t);
    }
  integsfr  *= age_step ;
  integsfr /= (age-age_min);
  double ssfr = integsfr*1e-6/stell_mass.Value(age);
   if ( ssfr > U_SSFR_MIN_VALUE )
      ssfr = log10(ssfr);
   else
     ssfr = SSFR_MIN_VALUE  ;
   return(ssfr); 

}

double GalaxyTemplate::sSFR(double age) const
{
  if (Has_SFR() && Has_Stellar_Mass() )
    {
      double ssfr = Specific_SFR(SFR,Stellar_Mass, age);
      return(ssfr);
    }
  else
    {
      return SSFR_BAD_VALUE ;
    }
}

==> fetch.php?media=snfit-2.4.1.tar.gz: snfit-2.4.2/src/galaxytemplate.cc <==
#include <cmath>

#include "galaxytemplate.h"
#include "datacards.h"
#include "pathutils.h"
#include "fileutils.h"
#include "snfitexception.h"

using namespace std;


 
GalaxyTemplate::GalaxyTemplate(const string& cardfilename) {
   LoadTemplates(cardfilename);
   if (Has_Star_Lum_Metallicity()) 
    cerr << "star lum metallicity" <<  endl ; 
   else 
    cerr << "NO star lum metallicity" <<  endl ; 
   if (Has_Gas_Metallicity()) 
    cerr << "gas metallicity" <<  endl ; 
   else 
    cerr << "NO gas metallicity" <<  endl ; 
   if (Has_SFR()) 
     cerr << "SFR" <<  endl ; 
   else 
     cerr << "NO SFR" <<  endl ;
   if (Has_Stellar_Mass()) 
     cerr << "stellar mass" <<  endl ; 
   else 
     cerr << "NO stellar mass" <<  endl ;
   if (Has_Stellar_WD_Mass()) 
     cerr << "stellar WD mass" <<  endl ; 
   else 
     cerr << "NO stellar WD mass" <<  endl ;
   if (Has_Stellar_Age_Mass()) 
     cerr << "stellar age average on  mass" <<  endl ; 
   else 
     cerr << "NO stellar age average on  mass" <<  endl ;
   if (Has_Stellar_Age_Lum()) 
     cerr << "stellar age average on lum" <<  endl ; 
   else 
     cerr << "NO stellar age average on lum" <<  endl ;
   if (Has_Tau_V()) 
     cerr << "optical depth in V" <<  endl ; 
   else 
     cerr << "NO optical depth in V" <<  endl ;
  cout << "INFO GalaxyTemplate::GalaxyTemplate done " << endl;
}


extern "C" {
  double divdifd_(const double *, const double *, const int *, const double *, const int*);
}

/*
static double HighDegreeInterpolationValue(const double& wavelength, const double& gal_type, const std::map<int,OneGalaxyTemplate>& templates) {
  int n = templates.size();
  double *xval = new double[n];
  double *fval = new double[n];
  int c = 0;
  for(std::map<int,OneGalaxyTemplate>::const_iterator it = templates.begin(); it != templates.end(); it++, c++) {
    xval[c] = it->first;
    fval[c] = log(it->second.Value(wavelength));
  }
  int interp_degree = 3;
  return exp( divdifd_(fval, xval, &n, &gal_type, &interp_degree) );
}
*/

/*
static void ResampleUsingInterpolation(Grid2DFunction& func) {
  
  double type_min = func.Xmin();
  double type_max = func.Xmax();
  
  size_t oldNx = func.NstepX();
  size_t oldNy = func.NstepY();
  
  // first get ymin and ymax from trimming 
  // i.e. discarding y values on boundaries for which all vals(x,y)=0
  size_t iymin = 0;
  while(true) {
    bool nonzero = false;
    for(size_t ix=0;ix<oldNx;ix++) {
      if(func.val(ix,iymin)!=0) {
	nonzero = true;
	break;
      }
    }
    if(nonzero) break;
    iymin++;
  }
  size_t iymax = oldNy-1;
  while(true) {
    bool nonzero = false;
    for(size_t ix=0;ix<oldNx;ix++) {
      if(func.val(ix,iymax)!=0) {
	nonzero = true;
	break;
      }
    }
    if(nonzero) break;
    iymax--;
  }
  
  cout << "INFO ResampleUsingInterpolation trimming gives Yrange  " << func.Yval(iymin) << " " << func.Yval(iymax) << endl;
  cout << "INFO ResampleUsingInterpolation initial Yrange         " << func.Yval(0) << " " << func.Yval(oldNy-1) << endl;
  
  
  int newNy = (iymax-iymin+1);
  int newNx = (type_max-type_min)*4; // to be tuned
  
  
  double *XValues  = new double[newNx];
  double *YValues  = new double[newNy];
  double **FValues = new double*[newNx];
  
  double *FValuesAtY = new double[oldNx];

  int interp_degree = 3;
  
  for(int j =0 ;j<newNy;j++) {
    YValues[j] = func.Yval(j+iymin);
  }


  for(int i=0;i<newNx;i++) {
    
    FValues[i] = new double[newNy];
    XValues[i] = type_min + i*(type_max-type_min)/(newNx-1);
    
    for(int j =0 ;j<newNy;j++) {
      
      int jinitial = j+iymin;
      // get min val to know scale
      double minval = 1.e60;
      for(size_t k=0;k<oldNx;k++) {
	const double &f = func.val(k,jinitial);
	if(f>0 && f<minval) minval=f;
      }
      
      for(size_t k=0;k<oldNx;k++) {
	if(func.val(k,jinitial)==0) // use log here
	  FValuesAtY[k] = log(minval*1.e-8);
	else
	  FValuesAtY[k] = log(func.val(k,jinitial)); 
      }
      int int_oldNx = oldNx;
      FValues[i][j] = exp( divdifd_(FValuesAtY, func.xvals(), &int_oldNx, &XValues[i], &interp_degree) );

    }
  }
  func = Grid2DFunction(newNx,newNy,(const double **)FValues,(const double *)XValues,(const double *)YValues);
  
  
  
}
*/


void GalaxyTemplate::LoadTemplates(const string& cardfilename) {
  has_sfr = false ;
  has_stellar_mass = false ;
  has_stellar_wd_mass = false ;
  has_gas_metallicity = false ;
  has_star_lum_metallicity = false ;

  bool bavard = true ;
  if ( getenv ("PLEASE_BE_QUIET" ) )
    bavard = false ;

  if (bavard)
    cout << "INFO GalaxyTemplate::LoadTemplates reading " << cardfilename << endl;
  string filename = cardfilename;
  string dirname = "";
  
  if(filename=="") {
    dirname  =  LocateFileFromCard("GALAXYTEMPLATES");
    filename = dirname+"/templates.cards";
  }else{
    dirname = DirName(filename);
  }
  if (bavard)
    cerr << "LoadTemplates : reading " << filename << endl ;
  FILE * file = fopen(filename.c_str(),"r");
  if(!file) {
    cerr << "fatal error in GalaxyTemplate::LoadTemplates(), cannot read file " << filename << endl;
    exit(-12);
  }
  double index;
  char type[512];
  char continuum_filename[512];
  char lines_filename[512];
  


  char line[4096] ;
  while (fgets(line,4096,file))
    {
      char *start = line;
      
      if (*start == '@') 
	{
	  char globkey[500];
	  char filename_key[1000] ;
	  if (sscanf(line, "@%s %s",globkey, filename_key) == 2 )
	    {  
	      if (bavard)
		cerr << "reading " << globkey << " " << filename_key << endl ;
	      string sglobkey = globkey ;
	      if ( sglobkey == "SFR_FILE" )
		{
		   
		  if (bavard) cerr << "SFR file " << filename_key << endl ;
		  General1DFunction func(dirname+"/"+string(filename_key));
		  SFR = func ;
		  has_sfr = true ;
		}
	      if ( sglobkey == "STAR_LUM_METALLICITY_FILE" )
		{
		  if (bavard) cerr << "STAR_LUM_METALLICITY file " << filename_key << endl ;
		  General1DFunction func(dirname+"/"+string(filename_key));
		  Star_Lum_Metallicity = func ;
		  has_star_lum_metallicity = true ;
		}
	      if ( sglobkey == "GAS_METALLICITY_FILE" )
		{
		  if (bavard) cerr << "GAS_METALLICITY file " << filename_key << endl ;
		  General1DFunction func(dirname+"/"+string(filename_key));
		  Gas_Metallicity = func ;
		  has_gas_metallicity = true ;
		}
	      if ( sglobkey == "STELLAR_MASS_FILE" )
		{
		  if (bavard) cerr << "stellar mass file " << filename_key << endl ;
		  General1DFunction func(dirname+"/"+string(filename_key)); 
		  Stellar_Mass = func ;
		  has_stellar_mass = true ;
	      
		}
	      if ( sglobkey == "STELLAR_WD_MASS_FILE" )
		{
		  if (bavard) cerr << "stellar wd mass file " << filename_key << endl ;
		  General1DFunction func(dirname+"/"+string(filename_key)); 
		  Stellar_WD_Mass = func ;
		  has_stellar_wd_mass = true ;
	      
		}
	      if ( sglobkey == "TAUV_FILE" )
		{
		  if (bavard) cerr << "tau v file " << filename_key << endl ;
		  General1DFunction func(dirname+"/"+string(filename_key)); 
		  Tau_V = func ;
		  has_tauv = true ;
	      
		}
	      if ( sglobkey == "STELL_AGES_MASS_FILE" )
		{
		  if (bavard) cerr << "stellar age mass file " << filename_key << endl ;
		  General1DFunction func(dirname+"/"+string(filename_key)); 
		  Stellar_Age_Mass = func ;
		  has_stellar_age_mass = true ;
	      
		}
	      if ( sglobkey == "STELL_AGES_LUM_FILE" )
		{
		  if (bavard) 
		    cerr << "stellar age lum file " << filename_key << endl ;
		  General1DFunction func(dirname+"/"+string(filename_key)); 
		  Stellar_Age_Lum = func ;
		  has_stellar_age_lum = true ;
	      
		}
	    }
	}
      else
	{
	  if (sscanf(line,"%lf %s %s %s",&index,type,continuum_filename,lines_filename) == 4)
	    {
	      if (bavard) cout << index << " " << type << " " << continuum_filename << " " << lines_filename << endl;

	      string uu = lines_filename ;
	      if ( uu == "none")
		{
		  HasLineSpectrum = false ;
		  cerr << "No lines provided " << endl ;
		}
	      else
		HasLineSpectrum = true ;

	      {
		General1DFunction func(dirname+"/"+string(continuum_filename));
		continuum.add_slice(index,func.values(),func.xvals(),func.Nval());
	      }
	      if (HasLineSpectrum)
		{
		  General1DFunction func(dirname+"/"+string(lines_filename));
		  lines.add_slice(index,func.values(),func.xvals(),func.Nval());
		}
	    }
	}
    }

  fclose(file);
  
  // don't resample any more
  //ResampleUsingInterpolation(continuum);
  //ResampleUsingInterpolation(lines);

  cout << "INFO GalaxyTemplate::LoadTemplates done " << endl;
}

Grid2DXSlice GalaxyTemplate::ContinuumSpectrum(const double& Type) const
{
  if(isnan(Type)) {
    cout << "ERROR GalaxyTemplate::ContinuumSpectrum Type is NaN" << endl;
    abort();
    throw(SnfitException("GalaxyTemplate::ContinuumSpectrum Type isNan")); 
  }
  if(Type<continuum.Xmin() || Type>continuum.Xmax()) {
    cout << "ERROR GalaxyTemplate::ContinuumSpectrum Type " << Type << 
      " out of range " << continuum.Xmin() << " " << continuum.Xmax() << endl;
    abort();
    throw(SnfitException("GalaxyTemplate::ContinuumSpectrum Type out of range"));
  } 
  return Grid2DXSlice(continuum,Type);
}

General1DFunction GalaxyTemplate::LineSpectrum(const double& Type) const
{
  // very clean handling of Kinney Spectra that come without lines
  if (! HasLineSpectrum)
    {
      double xval[2]={1000.,2000.};
      double fval[2]={0.,0.};
      General1DFunction null_func(2,&fval[0],&xval[0]);
      return null_func ;
    }


  if(isnan(Type)) {
    cout << "ERROR GalaxyTemplate::LineSpectrum Type is NaN" << endl;
    abort();
    throw(SnfitException("GalaxyTemplate::LineSpectrum Type isNan")); 
  }
  if(Type<lines.Xmin() || Type>lines.Xmax()) {
    cout << "ERROR GalaxyTemplate::LineSpectrum Type " << Type << 
      " out of range " << lines.Xmin() << " " << lines.Xmax() << endl;
    abort();
    throw(SnfitException("GalaxyTemplate::LineSpectrum Type out of range"));
  }
 
  size_t Nx = lines.NstepY(); // c'est pas un bug
  const double *xvals = lines.yvals(); // c'est les lambda (x = age)
  const double *pxdeb = xvals ;


  Vect& buffer = const_cast<Vect&>(lines_fvals_buffer);
  if(buffer.size()!=Nx)
    buffer.allocate(Nx);
  
  double *fvals = buffer.NonConstData();
  double *pfdeb = fvals ;

  for(size_t i=0;i<Nx;i++,xvals++,fvals++)
    *fvals = lines.Value(Type,*xvals);
  //General1DFunction func(Nx,&fvals[0],&xvals[0]);
  General1DFunction func(Nx,pfdeb,pxdeb);
  
  return func;
}


double GalaxyTemplate::Value(const double& wavelength, const double& gal_type) const {
  return continuum.Value(gal_type,wavelength);
  // return ContinuumSpectrum(gal_type).Value(wavelength); 
  // Grid2DXSlice(continuum,Type);
  // sampledfunction.h : Grid2DXSlice(const Grid2DFunction &F, const double Xval) ;
  // Grid2DXSlice.Value(Xwhere) = Parent2DFunction.Value(xVal, XWhere);}
  // so it is indeed :  (continuum.Value(Type, wavelength);
}


static double Specific_SFR(const  General1DFunction & sfr, const  General1DFunction  & stell_mass, const double  age)
{ 

 
  // integrating between  age-500 Myr and  age.
  double age_min = age-500 ;
  if (age_min < 0)
    age_min = 1. ;
  double Nage = 100 ;
  double age_step = (age-age_min)/Nage ;
  double integsfr = 0 ;
  for(int i = 0 ; i <= Nage ; i++)
    {
      double t = age_min + i*age_step ;
      integsfr += sfr.Value(t);
    }
  integsfr  *= age_step ;
  integsfr /= (age-age_min);
  double ssfr = integsfr*1e-6/stell_mass.Value(age);
   if ( ssfr > U_SSFR_MIN_VALUE )
      ssfr = log10(ssfr);
   else
     ssfr = SSFR_MIN_VALUE  ;
   return(ssfr); 

}


double GalaxyTemplate::StellarMass(double age) const
{ 
  double u = -1 ;
  if (Has_Stellar_Mass() )
    u = Stellar_Mass.Value(age) ;
  else
    cerr << "WARNING GalaxyTemplate::StellarMass not available for this template" << endl ;
  return(u);
}
double GalaxyTemplate::StellarWDMass(double age) const
{ 
  double u = -1 ;
  if (Has_Stellar_WD_Mass() )
    u = Stellar_WD_Mass.Value(age) ;
  else
    cerr << "WARNING GalaxyTemplate::StellarWDMass not available for this template" << endl ;
  return(u);
}

double GalaxyTemplate::S_F_R(double age) const
{ 
  double u = -1 ;
  if (Has_SFR() )
    u = SFR.Value(age) ;
  else
    cerr << "WARNING GalaxyTemplate::S_F_R not available for this template" << endl ;
  return(u);
}


double GalaxyTemplate::sSFR(double age) const
{
  if (Has_SFR() && Has_Stellar_Mass() )
    {
      double ssfr = Specific_SFR(SFR,Stellar_Mass, age);
      return(ssfr);
    }
  else
    {
      return SSFR_BAD_VALUE ;
    }
}

double GalaxyTemplate::Mean_Stellar_Age_Mass(double age) const
{ 
  double u = -1 ;
  if (Has_Stellar_Age_Mass() )
    u = Stellar_Age_Mass.Value(age) ;
  else
    cerr << "WARNING GalaxyTemplate::Mean_Stellar_Age_Mass not available for this template" << endl ;
  return(u);
}

double GalaxyTemplate::Mean_Stellar_Age_Lum(double age) const
{ 
  double u = -1 ;
  if (Has_Stellar_Age_Lum() )
    u = Stellar_Age_Lum.Value(age) ;
  else
    cerr << "WARNING GalaxyTemplate::Mean_Stellar_Age_Lum not available for this template" << endl ;
  return(u);
}

  double GalaxyTemplate::GasMetallicity(double age) const 
{ 
  double u = -1 ;
  if (Has_Gas_Metallicity() )
    u = Gas_Metallicity.Value(age) ;
  else
    cerr << "WARNING GalaxyTemplate::Gas_Metallicity not available for this template" << endl ;
  return(u);
}

  double GalaxyTemplate::StarLumMetallicity(double age) const 
{ 
  double u = -1 ;
  if (Has_Star_Lum_Metallicity() )
    u = Star_Lum_Metallicity.Value(age) ;
  else
    cerr << "WARNING GalaxyTemplate::Star_Lum_Metallicity not available for this template" << endl ;
  return(u);
}

double GalaxyTemplate::TauV(double age) const 
{ 
  double u = -1 ;
  if (Has_Tau_V() )
    u = Tau_V.Value(age) ;
  else
    cerr << "WARNING GalaxyTemplate::Tau_V not available for this template" << endl ;
  return(u);
}
